﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PuferFishContracts.DataModels;
using PuferFishContracts.Exceptions;

namespace PuferFishTests.DataModelsTests
{
    [TestFixture]
    internal class BuyerDataModelTests
    {
        [Test]
        public void IdIsNullOrEmptyTest()
        {
            var buyer = CreateDataModel(null, "Фамилия Имя Отчество", "number", 10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
            buyer = CreateDataModel(string.Empty, "Фамилия Имя Отчество", "number", 10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
        }
        [Test]
        public void IdIsNotGuidTest()
        {
            var buyer = CreateDataModel("id", "Фамилия Имя Отчество", "number", 10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
        }
        [Test]
        public void FIOIsNullOrEmptyTest()
        {
            var buyer = CreateDataModel(Guid.NewGuid().ToString(), null,
            "number", 10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
            buyer = CreateDataModel(Guid.NewGuid().ToString(), string.Empty,
            "number", 10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
        }
        [Test]
        public void PhoneNumberIsNullOrEmptyTest()
        {
            var buyer = CreateDataModel(Guid.NewGuid().ToString(), "Фамилия Имя Отчество", null,
            10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
            buyer = CreateDataModel(Guid.NewGuid().ToString(), "Фамилия Имя Отчество",
            string.Empty, 10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
        }
        [Test]
        public void PhoneNumberIsIncorrectTest()
        {
            var buyer = CreateDataModel(Guid.NewGuid().ToString(), "Фамилия Имя Отчество", "777",
            10);
            Assert.That(() => buyer.Validate(),
            Throws.TypeOf<ValidationException>());
        }
        [Test]
        public void AllFieldsIsCorrectTest()
        {
            var buyerId = Guid.NewGuid().ToString();
            var fio = "Фамилия Имя Отчество";
            var phoneNumber = "+7-777-777-77-77";
            var points = 11;
            var buyer = CreateDataModel(buyerId, fio, phoneNumber, points);
            Assert.That(() => buyer.Validate(), Throws.Nothing);
            Assert.Multiple(() =>
            {
                Assert.That(buyer.Id, Is.EqualTo(buyerId));
                Assert.That(buyer.FIO, Is.EqualTo(fio));
                Assert.That(buyer.PhoneNumber, Is.EqualTo(phoneNumber));
                Assert.That(buyer.Points, Is.EqualTo(points));
            });
        }
        private static BuyerDataModel CreateDataModel(string? id, string? fio, string? phoneNumber, double points) => new(id, fio, phoneNumber, points);
    }
}
